@@ -21,6 +21,17 @@ from oslo_log import log as logging
 from horizon.utils import functions as utils
 from horizon.utils.memoized import memoized  # noqa
 
+from keystoneauth1 import loading
+from keystoneauth1 import session
+from novaclient import client as nova_client
+
+# Supported compute versions
+NOVA_VERSIONS = base.APIVersionManager("compute", preferred_version=2)
+NOVA_VERSIONS.load_supported_version(1.1,
+                                     {"client": nova_client, "version": 1.1})
+NOVA_VERSIONS.load_supported_version(2, {"client": nova_client, "version": 2})
+NOVA_VERSION = NOVA_VERSIONS.get_active_version()['version']
+
 LOG = logging.getLogger(__name__)
 
 
@@ -133,20 +144,27 @@ def instance_delete(request, instance_id):
     return troveclient(request).instances.delete(instance_id)
 
 
-def instance_create(request, name, volume, flavor, databases=None,
+def instance_create(request, name, volume, flavor=None, databases=None,
                     users=None, restore_point=None, nics=None,
                     datastore=None, datastore_version=None,
                     replica_of=None, replica_count=None,
                     volume_type=None, configuration=None, locality=None,
-                    availability_zone=None):
+                    availability_zone=None, access=None):
     # TODO(dklyle): adding conditional to support trove without volume
     # support for now until API supports checking for volume support
-    if volume > 0:
+    if volume > 0 and not replica_of:
         volume_params = {'size': volume}
         if volume_type:
             volume_params['type'] = volume_type
     else:
         volume_params = None
+
+    if replica_of:
+        flavor = None
+        volume_params = None
+        datastore = None
+        datastore_version = None
+
     return troveclient(request).instances.create(
         name,
         flavor,
@@ -161,7 +179,8 @@ def instance_create(request, name, volume, flavor, databases=None,
         replica_count=replica_count,
         configuration=configuration,
         locality=locality,
-        availability_zone=availability_zone)
+        availability_zone=availability_zone,
+        access=access)
 
 
 def instance_resize_volume(request, instance_id, size):
@@ -181,9 +200,13 @@ def instance_restart(request, instance_id):
     return troveclient(request).instances.restart(instance_id)
 
 
+def instance_update(request, instance_id, **kwargs):
+    return troveclient(request).instances.update(instance_id, **kwargs)
+
+
 def instance_detach_replica(request, instance_id):
-    return troveclient(request).instances.edit(instance_id,
-                                               detach_replica_source=True)
+    return troveclient(request).instances.update(instance_id,
+                                                 detach_replica_source=True)
 
 
 def promote_to_replica_source(request, instance_id):
@@ -235,30 +258,56 @@ def backup_delete(request, backup_id):
 
 
 def backup_create(request, name, instance_id, description=None,
-                  parent_id=None):
+                  parent_id=None, swift_container=None):
     return troveclient(request).backups.create(name, instance_id,
-                                               description, parent_id)
+                                               description=description,
+                                               parent_id=parent_id,
+                                               swift_container=swift_container)
+
+
+def backup_strategy_create(request, instance_id=None, swift_container=None):
+    return troveclient(request).backup_strategies.create(
+        instance_id=instance_id, swift_container=swift_container)
+
+
+def backup_strategy_list(request, instance_id=None, project_id=None):
+    return troveclient(request).backup_strategies.list(instance_id=instance_id,
+                                                       project_id=project_id)
+
+
+def backup_strategy_delete(request, instance_id=None, project_id=None):
+    return troveclient(request).backup_strategies.delete(
+        instance_id=instance_id, project_id=project_id)
+
+
+def nova_client_client(request):
+    insecure = getattr(settings, 'OPENSTACK_SSL_NO_VERIFY', False)
+    cacert = getattr(settings, 'OPENSTACK_SSL_CACERT', None)
+    identity_url = base.url_for(request, 'identity')
+    loader = loading.get_plugin_loader('token')
+    auth = loader.load_from_options(auth_url=identity_url,
+                                    token=request.user.token.id,
+                                    tenant_id=request.user.project_id)
+    sess = session.Session(auth=auth)
+    nova = nova_client.Client(NOVA_VERSION,
+                              session=sess,
+                              insecure=insecure,
+                              cacert=cacert,
+                              http_log_debug=settings.DEBUG)
+    return nova
 
 
 def flavor_list(request):
-    return troveclient(request).flavors.list()
+    return nova_client_client(request).flavors.list()
 
 
 def datastore_flavors(request, datastore_name=None,
                       datastore_version=None):
-    # if datastore info is available then get datastore specific flavors
-    if datastore_name and datastore_version:
-        try:
-            return troveclient(request).flavors.\
-                list_datastore_version_associated_flavors(datastore_name,
-                                                          datastore_version)
-        except Exception:
-            LOG.warning("Failed to retrieve datastore specific flavors")
     return flavor_list(request)
 
 
 def flavor_get(request, flavor_id):
-    return troveclient(request).flavors.get(flavor_id)
+    return nova_client_client(request).flavors.get(flavor_id)
 
 
 def root_enable(request, instance_ids):
@@ -400,3 +449,7 @@ def configuration_update(request, group_id, values):
 
 def configuration_default(request, instance_id):
     return troveclient(request).instances.configuration(instance_id)
+
+
+def stop_database(request, instance_id):
+    return troveclient(request).mgmt_instances.stop(instance_id)
