@@ -17,12 +17,10 @@ Views for managing database instances.
 """
 from collections import OrderedDict
 
-from django.core.urlresolvers import reverse
-from django.core.urlresolvers import reverse_lazy
+from django.urls import reverse
+from django.urls import reverse_lazy
 from django.utils.translation import ugettext_lazy as _
 
-import six
-
 from horizon import exceptions
 from horizon import forms as horizon_forms
 from horizon import tables as horizon_tables
@@ -58,7 +56,7 @@ class IndexView(horizon_tables.DataTableView):
             flavors = []
             msg = _('Unable to retrieve database size information.')
             exceptions.handle(self.request, msg)
-        return OrderedDict((six.text_type(flavor.id), flavor)
+        return OrderedDict((str(flavor.id), flavor)
                            for flavor in flavors)
 
     def _extra_data(self, instance):
@@ -80,7 +78,8 @@ class IndexView(horizon_tables.DataTableView):
             instances = []
             msg = _('Unable to retrieve database instances.')
             exceptions.handle(self.request, msg)
-        map(self._extra_data, instances)
+        for instance in instances:
+            self._extra_data(instance)
         return instances
 
 
@@ -96,6 +95,43 @@ class LaunchInstanceView(horizon_workflows.WorkflowView):
         return initial
 
 
+class UpdateInstanceView(horizon_forms.ModalFormView):
+    form_class = forms.UpdateInstanceForm
+    form_id = "attach_config_form"
+    modal_header = _("Update Instance")
+    modal_id = "edit_instance_modal"
+    template_name = "project/databases/update_instance.html"
+    submit_label = "Update"
+    submit_url = 'horizon:project:databases:edit_instance'
+    success_url = reverse_lazy('horizon:project:databases:index')
+
+    @memoized.memoized_method
+    def get_object(self, *args, **kwargs):
+        instance_id = self.kwargs['instance_id']
+        try:
+            return api.trove.instance_get(self.request, instance_id)
+        except Exception:
+            msg = _('Unable to retrieve instance details.')
+            redirect = reverse('horizon:project:databases:index')
+            exceptions.handle(self.request, msg, redirect=redirect)
+
+    def get_context_data(self, **kwargs):
+        context = (super(UpdateInstanceView, self)
+                   .get_context_data(**kwargs))
+        context['instance_id'] = self.kwargs['instance_id']
+        args = (self.kwargs['instance_id'],)
+        context['submit_url'] = reverse(self.submit_url, args=args)
+        return context
+
+    def get_initial(self):
+        instance = self.get_object()
+        allowed_cidrs = instance.access.get('allowed_cidrs', ['0.0.0.0/0'])
+        return {'instance_id': self.kwargs['instance_id'],
+                'instance_name': instance.name,
+                'allowed_cidrs': ','.join(allowed_cidrs),
+                'datastore_version': instance.datastore.get('version', '')}
+
+
 class DBAccess(object):
     def __init__(self, name, access):
         self.name = name
